needed_packages <- c(
  "shiny"
, "ggplot2"
, "deSolve"
, "dplyr"
, "reshape2"
, "shinythemes"
, "shinydashboard"
)

## Check if the packages are installed. If they are not install them, then load them
if (length(setdiff(needed_packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(needed_packages, rownames(installed.packages())))  
}

lapply(needed_packages, require, character.only = TRUE)

## Not yet added 
  # input$mos_temp_uncer_runs
  # input$mos_human_uncer_runs

function(input, output) {
  
  source("ggplot_theme.R")
  source("NatCap_VBD_Functions.R")
  source("NatCap_VBD_epi_model.R")
  
  ## Load some data whether we need to or not so that we don't have to keep reloading it when input changes
 ## Disease parameters
Epi_parameters    <- read.csv("data/Epi_parameters.csv")
 ## Temperate data
mosq_temp_params  <- read.csv("data/mosquito_temp_params.csv")
 ## Template for the parameters needed
aedes_temp_params <- read.csv("data/aedes_temp_params.csv")

  ### Some parameters that -- at least for now -- I don't give user's access to:
  mu_h           <- 1/(61*365)
  uncer.start    <- "none"
  uncer.start.sd <- 0.1
  
  epi.dat <- reactive({
     
  ### Model Parameters:
  N_h <- input$N_0         ## Total human population
  I_h <- input$I_h0 * N_h  ## Humans infected
  R_h <- input$R_h0 * N_h  ## Humans recovered
  S_h <- N_h - I_h - R_h   ## Humans susceptible
  np  <- input$np          ## Time periods

########################################
### Temperature dependent parameters ###
########################################
if (input$mos_temp_dep == TRUE) {

## Goal is do to some sort of averaging over parameters based on a users chosen options for mosquito species and disease
supp_spec    <- c("Aedes", "Culex", "Culiseta") 

## (Step 1): Identify the genus
spec_id      <- which(apply(data.frame(mosq_species = supp_spec), 1
  , FUN = function (x) ifelse(length(grep(x, input$mosq_species)) == 1, 1, 0)) == 1)
mosq_genus   <- supp_spec[spec_id]
if (length(mosq_genus) < 1) {
mosq_genus   <- "Unknown"  
}

## (Step 2): Identify the species
no_spec      <- ifelse(length(grep(" sp", input$mosq_species, ignore.case = TRUE)) == 1, 1, 0)
## if user doesn't specify species, determine the species
if (no_spec == 0) {
mosq_spec    <- strsplit(input$mosq_species, " ")[[1]][2]
} else {
mosq_spec    <- "sp."
}

## (Step 3): Extract all needed parameters for this mosquito. Store as many parameter values as needed given certainty with
 ## mosquito, climate and disease
all_params   <- unique(mosq_temp_params$Trait)

## Note: we don't want to be averaging parameters at this stage, but saving all of the fitted curve parameter
 ## values, and averaging the estimated response (due to different functions fit for different mosquitoes)
## *!* A key next step will be to better incorporate the temperate vs tropical into more parameters
for (i in seq_along(all_params)) {
 
## Find the parameter value to use for the specified mosquito species (and maybe disease)
temp_dat <- mosq_temp_params %>% filter(Trait == all_params[i])

## Lifespan stands out as using different parameters than all other traits
 ## b and c stand out as being disease specific
if (temp_dat$Trait[1] != "lf" & temp_dat$Trait[1] != "b" & temp_dat$Trait[1] != "c") {

## Three steps:
 ## (1) Look for a perfect match
param_val <- temp_dat %>% filter(Mosquito.Genus == mosq_genus & Mosquito.Species == mosq_spec)
 ## (2) If no match look for just genus
if (nrow(param_val) == 0) {
param_val <- temp_dat %>% filter(Mosquito.Genus == mosq_genus)  
}
#if (nrow(param_val) == 1) {
#param_val[, 3] <- "Avg"  
#}
 ## (2.1) If there is a match and multiple rows of data, average parameters
#if (nrow(param_val) > 1) {
#param_val[1, -(1:5)] <- colMeans(param_val[, -(1:5)])
#param_val            <- param_val[1, ]
#param_val[, 3]       <- "Avg"
#}
 ## (3) If still no match average all rows
if (nrow(param_val) == 0) {
param_val <- temp_dat
#param_val[1, -(1:5)] <- colMeans(param_val[, -(1:5)])
#param_val            <- param_val[1, ]
#param_val[, c(2, 3)] <- c("Avg", "Avg")
}
  
} else if (temp_dat$Trait[1] == "lf") {
  
## Three steps:
 ## (1) Look for a perfect match
param_val <- temp_dat %>% filter(Mosquito.Genus == mosq_genus & Mosquito.Species == mosq_spec)
 ## (2) If no match look for just genus
if (nrow(param_val) == 0) {
param_val <- temp_dat %>% filter(Mosquito.Genus == mosq_genus)  
}
#if (nrow(param_val) == 1) {
#param_val[, 3] <- "Avg"  
#}
 ## (2.1) If there is a match and multiple rows of data, first look for climate, then average parameters if there are still multiple rows
#if (nrow(param_val) > 1) {
#param_val <- param_val %>% filter(Climate == temp_depend)
#} 
#if (nrow(param_val) > 1) {
#param_val[1, -(1:5)] <- colMeans(param_val[, -(1:5)])
#param_val            <- param_val[1, ]
#param_val[, 3] <- "Avg"
#}
 ## (3) If still no match average all rows of the right climate
if (nrow(param_val) == 0) {
param_val <- temp_dat %>% filter(Climate == input$temp_loc)
#param_val[1, -(1:5)] <- colMeans(param_val[, -(1:5)])
#param_val            <- param_val[1, ]
#param_val[, c(2, 3)] <- c("Avg", "Avg")
}

} else {
  
## More than three steps because of the extra column to search over:
 ## (1) Look for a perfect match
param_val <- temp_dat %>% filter(Mosquito.Genus == mosq_genus & Mosquito.Species == mosq_spec & Disease == input$dis_type)
 ## (2) If no match look for just genus
if (nrow(param_val) == 0) {
param_val <- temp_dat %>% filter(Mosquito.Genus == mosq_genus & Disease == input$dis_type)  
}
#if (nrow(param_val) == 1) {
#param_val[, 3] <- "Avg"  
#}
 ## (3) If still no match, just look for the right disease
 ## *!* This choice is operating under the assumption that the disease holds more variance than the mosquito for b and c (unclear about this)
if (nrow(param_val) == 0) {
param_val <- temp_dat %>% filter(Disease == input$dis_type)  
}
#if (nrow(param_val) == 1) {
#param_val[, c(2, 3)] <- c("Avg", "Avg") 
#}
 ## (3.1) If there is a match and multiple rows of data, average parameters
#if (nrow(param_val) > 1) {
#param_val[1, -(1:5)] <- colMeans(param_val[, -(1:5)])
#param_val            <- param_val[1, ]
#param_val[, c(2, 3)] <- c("Avg", "Avg") 
#}
 ## (4) If still no match average all rows
if (nrow(param_val) == 0) {
param_val <- temp_dat
#param_val[1, -(1:5)] <- colMeans(param_val[, -(1:5)])
#param_val            <- param_val[1, ]
#param_val[, c(2, 3)] <- c("Avg", "Avg")
}
  
}

## Store the parameter values for this parameter
if (i == 1) {
temp_params   <- param_val
} else {
temp_params.t <- param_val
temp_params   <- rbind(temp_params, temp_params.t)
}
  
}

}
  
###################################
### Disease dependent parameters###
###################################
  
if (input$dis_type != "None") {
  
beta_hm   <- Epi_parameters[Epi_parameters$Parameter == "beta_hm" & Epi_parameters$Disease == input$dis_type, ]$est 
alpha_h   <- Epi_parameters[Epi_parameters$Parameter == "alpha_h" & Epi_parameters$Disease == input$dis_type, ]$est
eta_h     <- Epi_parameters[Epi_parameters$Parameter == "eta_h"   & Epi_parameters$Disease == input$dis_type, ]$est
gamma_m   <- Epi_parameters[Epi_parameters$Parameter == "gamma_m" & Epi_parameters$Disease == input$dis_type, ]$est
gamma_h   <- Epi_parameters[Epi_parameters$Parameter == "gamma_h" & Epi_parameters$Disease == input$dis_type, ]$est
psi_h     <- Epi_parameters[Epi_parameters$Parameter == "psi_h"   & Epi_parameters$Disease == input$dis_type, ]$est
omega_Ah  <- Epi_parameters[Epi_parameters$Parameter == "omega_h" & Epi_parameters$Disease == input$dis_type, ]$est
delta_h   <- Epi_parameters[Epi_parameters$Parameter == "delta_h" & Epi_parameters$Disease == input$dis_type, ]$est
omega_Eh  <- ifelse(input$dis_type == "Malaria", 1, 0)
omega_Em  <- ifelse(input$dis_type == "Malaria", 1, 0)

}
  
###########################################################
### set up a list of all the levels of any intervention ###
######################################## ##################   
if (input$intervention == "None") {
int.strength   <- c(1) 
} else {
int.strength   <- c(1, 2/3, 1/2, 1/3)
}
epi.params.all <- vector("list", length = length(int.strength))
  
#####################################
### Package all of the parameters ###
#####################################
### Most efficient to just wrap everything into a loop over the intervention options
for (j in seq_along(epi.params.all)) {

### A few options for how parameters are chosen:
  ## 1) No temperature or human dependence
  ## 2) Temperature dependent 
  ## 3) Human population dependent
  ## 4) Both

### Within each of these options, sensitivity/uncertainty can be turned on or off

###############################################################################
## Set up the temperature dependent parameter values if they are called for ###
###############################################################################
if (input$mos_temp_dep == TRUE) {
  
     ### Construct the correct temperature dependence for a few parameters for the chosen mosquito species and disease
      ## *!* Oct 1, 2019: Could consider incorporating the diversity of values as the uncertainty instead of taking the mean and then re-running
       ## with parameter values with some sd around the derived mean. Come back to this
     
     ### *!* Oct 1, 2019: I hate how long this section of code is, but I can't really think of a way to streamline it but also give the user as many
       ## options as i want to
     
     ### B ###
     tp.B <- temp_params %>% filter(Trait_F == "a")
     tp.B <- mean(with(tp.B, Briere_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")))
     
     ### eta_a ###
     tp.eta_a <- temp_params %>% filter(Trait_F == "MDR")
     tp.eta_a <- mean(with(tp.eta_a, Briere_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")))
  
     ### beta_mh ###
     tp.beta_mh <- temp_params %>% filter(Trait_F == "b")
     temp_val <- numeric(nrow(tp.beta_mh))
     for (k in 1:nrow(tp.beta_mh)) {
         temp_val[k] <- with(tp.beta_mh[k, ]
           , if (Function == "Briere") {
             Briere_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           } else {
             Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           }
         )
     }
     tp.beta_mh <- mean(temp_val)
     
     ### beta_hm ###
     tp.beta_hm <- temp_params %>% filter(Trait_F == "c")
     temp_val <- numeric(nrow(tp.beta_hm))
     for (k in 1:nrow(tp.beta_hm)) {
         temp_val[k] <- with(tp.beta_hm[k, ]
           , if (Function == "Briere") {
             Briere_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           } else {
             Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           }
           )
     }
     tp.beta_hm <- mean(temp_val)
     
     ### phi ###
      ## Components
     tp.phi.EFGC <- temp_params %>% filter(Trait == "ER" | Trait == "EFGC")
     tp.phi.pO   <- temp_params %>% filter(Trait == "pO")
      ## Direct estimate
     tp.phi.EFD  <- temp_params %>% filter(Trait == "EFD")
     
     temp_val.EFGC <- mean(with(tp.phi.EFGC, Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")))
   
     temp_val <- numeric(nrow(tp.phi.pO))
     for (k in 1:nrow(tp.phi.pO)) {
         temp_val[k] <- with(tp.phi.pO[k, ]
           , if (Function == "Briere") {
             Briere_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           } else {
             Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           }
           )
     }
     tp.phi.pO     <- mean(temp_val)
     ## Estimate from component parts
     tp.phi.f.comp <- temp_val.EFGC * tp.phi.pO
     ## And the final from the mean of this and that of the direct estimate
     tp.phi        <- mean(tp.phi.f.comp
       , mean(with(tp.phi.EFD, Briere_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no"))))   
    
     ### mu_a ###
      ## Components
     tp.mu_a.EV  <- temp_params %>% filter(Trait == "EV")
     tp.mu_a.pLA <- temp_params %>% filter(Trait == "pLA")
      ## Direct measurement
     tp.mu_a     <- temp_params %>% filter(Trait == "pEA")
     
     temp_val.pLA <- mean(with(tp.mu_a.pLA, Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")))
   
     temp_val <- numeric(nrow(tp.mu_a.EV))
     for (k in 1:nrow(tp.mu_a.EV)) {
         temp_val[k] <- with(tp.mu_a.EV[k, ]
           , if (Function == "Briere") {
             Briere_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           } else {
             Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")
           }
           )
     }
     tp.mu_a.EV     <- mean(temp_val)
     ## Estimate from component parts
     tp.mu_a.f.comp <- tp.mu_a.EV * temp_val.pLA
     ## And the final from the mean of this and that of the direct estimate
     tp.mu_a        <- mean(tp.mu_a.f.comp
       , mean(with(tp.mu_a, Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")))) 

     ### mu_m ###
     tp.mu_m <- temp_params %>% filter(Trait_F == "lf")
     temp_val <- numeric(nrow(tp.mu_m))
     for (k in 1:nrow(tp.mu_m)) {
         temp_val[k] <- with(tp.mu_m[k, ]
           , if (Function == "Quad") {
             (1/(Quad_curve(c = c_mean, tmin = tmin_mean, tmax = tmax_mean, temp = input$temp, parm = "no")*0.5))
           } else {
             1/(Linear_curve(m = m_est, z = z_est, temp = input$temp)*0.5)
           }
         )
     }
     tp.mu_m <- mean(temp_val)  
   
}
  
#############################################
### 1) No temperature or human dependence ###
#############################################

  if (input$mos_temp_dep == FALSE & input$mos_human_dep == FALSE) {
    
if (input$dis_type == "None") {
    
     epi.params <- c(
      mu_h     = mu_h 
    , B        = input$B    
    , beta_mh  = input$beta_mh      
    , phi      = input$phi
    , k        = input$k
    , eta_a    = input$eta_a     
    , mu_a     = input$mu_a
    , mu_m     = input$mu_m
    , m_per_h  = input$m_per_h
    , beta_hm  = input$beta_hm  
    , eta_h    = input$eta_h 
    , gamma_m  = input$gamma_m
    , gamma_h  = input$gamma_h
    , alpha_h  = input$alpha_h
    , psi_h    = input$psi_h
    , omega_Ah = input$omega_Ah
    , omega_Eh = as.numeric(input$omega_Eh)
    , omega_Em = as.numeric(input$omega_Em)
    , delta_h  = input$delta_h
    )
     
} else {
  
     epi.params <- c(
      mu_h     = mu_h 
    , B        = input$B    
    , beta_mh  = input$beta_mh      
    , phi      = input$phi
    , k        = input$k
    , eta_a    = input$eta_a     
    , mu_a     = input$mu_a
    , mu_m     = input$mu_m
    , m_per_h  = input$m_per_h
    , beta_hm  = beta_hm  
    , eta_h    = eta_h 
    , gamma_m  = gamma_m
    , gamma_h  = gamma_h
    , alpha_h  = alpha_h
    , psi_h    = psi_h
    , omega_Ah = omega_Ah
    , omega_Eh = omega_Eh
    , omega_Em = omega_Em
    , delta_h  = delta_h
     )
  
}
     
    epi.params <- as.data.frame(t(epi.params))
    
    ## Implement the intervention 
    ## *!* could consider some types of intervention that start later in the epidemic
    if (input$intervention != "None") {
        if (input$intervention == "mu_a" | input$intervention == "mu_m" | input$intervention == "eta_h") {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] / int.strength[j]
      } else {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] * int.strength[j]  
      }
    }
    
    ## Control of uncertainty/parameter variability ########
    if (input$uncer_type == "all") {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs * ncol(epi.params)), ]
    epi.params <- apply(epi.params, 2, function(x) rlnorm(length(x), log(x), input$uncer_logsd * x))
  
    } else if (input$uncer_type %in% names(epi.params) == TRUE) {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs), ]
    ## !*! Placeholder for now for variability/uncertainty
    epi.params[[input$uncer_type]] <- rlnorm(input$uncer_runs, log(epi.params[[input$uncer_type]])
      , input$uncer_logsd * epi.params[[input$uncer_type]])
    
    } else if (input$uncer_type == "none") {
      ## Do nothing      
    } else {
      return(list(c("ERROR with uncertainty specification: Not a real parameter choice")))
    }
   
#################################
### 2) Temperature dependence ###
#################################
     
   } else if (input$mos_temp_dep == TRUE & input$mos_human_dep == FALSE) {
     
if (input$dis_type == "None") {
     
    epi.params <- c(
      mu_h     = mu_h
    , B        = tp.B  
    , beta_mh  = tp.beta_mh
    , phi      = tp.phi
    , k        = input$k
    , eta_a    = tp.eta_a  
    , mu_a     = 1 - tp.mu_a
    , beta_hm  = tp.beta_hm 
    , mu_m     = tp.mu_m     ## Mosquitoes blow up in this model with the lab parameters so adjust a bit (drop by 50%)
    , m_per_h  = input$m_per_h
    , eta_h    = input$eta_h
    , gamma_m  = input$gamma_m
    , gamma_h  = input$gamma_h
    , alpha_h  = input$alpha_h
    , psi_h    = input$psi_h
    , omega_Ah = input$omega_Ah
    , omega_Eh = as.numeric(input$omega_Eh)
    , omega_Em = as.numeric(input$omega_Em)
    , delta_h  = input$delta_h
    )  
    
} else {
  
    epi.params <- c(
      mu_h     = mu_h
    , B        = tp.B  
    , beta_mh  = tp.beta_mh
    , phi      = tp.phi
    , k        = input$k
    , eta_a    = tp.eta_a  
    , mu_a     = 1 - tp.mu_a
    , beta_hm  = tp.beta_hm 
    , mu_m     = tp.mu_m     ## Mosquitoes blow up in this model with the lab parameters so adjust a bit (drop by 50%)
    , m_per_h  = input$m_per_h
    , eta_h    = input$eta_h
    , gamma_m  = gamma_m
    , gamma_h  = gamma_h
    , alpha_h  = alpha_h
    , psi_h    = psi_h
    , omega_Ah = omega_Ah
    , omega_Eh = omega_Eh
    , omega_Em = omega_Em
    , delta_h  = delta_h
    )  
  
}
     
    epi.params <- as.data.frame(t(epi.params))
    
    ## Implement the intervention 
    ## *!* could consider some types of intervention that start later in the epidemic
    if (input$intervention != "None") {
        if (input$intervention == "mu_a" | input$intervention == "mu_m" | input$intervention == "eta_h") {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] / int.strength[j]
      } else {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] * int.strength[j]  
      }
    }
    
    ## Control of uncertainty/parameter variability ########
    if (input$uncer_type == "all") {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs * ncol(epi.params)), ]
    epi.params <- apply(epi.params, 2, function(x) rlnorm(length(x), log(x), input$uncer_logsd * x))
  
    } else if (input$uncer_type %in% names(epi.params) == TRUE) {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs), ]
    ## !*! Placeholder for now for variability/uncertainty
    epi.params[[input$uncer_type]] <- rlnorm(input$uncer_runs, log(epi.params[[input$uncer_type]])
      , input$uncer_logsd * epi.params[[input$uncer_type]])
    
    } else if (input$uncer_type == "none") {
      ## Do nothing      
    } else {
      return(list(c("ERROR with uncertainty specification: Not a real parameter choice")))
    }
     
##############################################
### 3) Human population density dependence ###
##############################################
     
   } else if (input$mos_temp_dep == FALSE & input$mos_human_dep == TRUE) {
    
    ## Generate mosquito biting rate based on human density
      ## !*! Turn these parameters into parameters the user can edit
      ## Assume this is in per/hectare, and that mosquito's cover 100m, which makes their area
       ## /10000 in hectares. Calculate # of humans in each mosqutioes flight area 
      hum_per_mfa <- (N_h / city_area(N_h, 2.2)) * (pi * 100^2) / 10000
      ## now assume humans per flight area scales to max bite by some arbitrary process
       ## !*! update this to be more mechanistic later, but for now just say that mosquitoes
        ## seek and encounter 0.05
      
if (input$dis_type == "None") {
      
    epi.params <- c(
      mu_h     = mu_h
    , B        = ifelse(hum_per_mfa * 0.2 >= 0.8, 0.8, hum_per_mfa * 0.2)    
    , beta_mh  = input$beta_mh      
    , phi      = input$phi
    , k        = input$k
    , eta_a    = input$eta_a     
    , mu_a     = input$mu_a
    , mu_m     = input$mu_m
    , m_per_h  = input$m_per_h
    , beta_hm  = input$beta_hm  
    , eta_h    = input$eta_h     
    , gamma_m  = input$gamma_m
    , gamma_h  = input$gamma_h
    , alpha_h  = input$alpha_h
    , psi_h    = input$psi_h
    , omega_Ah = input$omega_Ah
    , omega_Eh = as.numeric(input$omega_Eh)
    , omega_Em = as.numeric(input$omega_Em)
    , delta_h  = input$delta_h
    )
    
} else {
  
    epi.params <- c(
      mu_h     = mu_h
    , B        = ifelse(hum_per_mfa * 0.2 >= 0.8, 0.8, hum_per_mfa * 0.2)    
    , beta_mh  = input$beta_mh      
    , phi      = input$phi
    , k        = input$k
    , eta_a    = input$eta_a     
    , mu_a     = input$mu_a
    , mu_m     = input$mu_m
    , m_per_h  = input$m_per_h
    , beta_hm  = beta_hm  
    , eta_h    = eta_h     
    , gamma_m  = gamma_m
    , gamma_h  = gamma_h
    , alpha_h  = alpha_h
    , psi_h    = psi_h
    , omega_Ah = omega_Ah
    , omega_Eh = omega_Eh
    , omega_Em = omega_Em
    , delta_h  = delta_h
    )  
  
}
     
    epi.params <- as.data.frame(t(epi.params))
    
    ## Implement the intervention 
    ## *!* could consider some types of intervention that start later in the epidemic
    if (input$intervention != "None") {
        if (input$intervention == "mu_a" | input$intervention == "mu_m" | input$intervention == "eta_h") {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] / int.strength[j]
      } else {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] * int.strength[j]  
      }
    }
    
    ## Control of uncertainty/parameter variability ########
    if (input$uncer_type == "all") {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs * ncol(epi.params)), ]
    epi.params <- apply(epi.params, 2, function(x) rlnorm(length(x), log(x), input$uncer_logsd * x))
  
    } else if (input$uncer_type %in% names(epi.params) == TRUE) {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs), ]
    ## !*! Placeholder for now for variability/uncertainty
    epi.params[[input$uncer_type]] <- rlnorm(input$uncer_runs, log(epi.params[[input$uncer_type]])
      , input$uncer_logsd * epi.params[[input$uncer_type]])
    
    } else if (input$uncer_type == "none") {
      ## Do nothing      
    } else {
      return(list(c("ERROR with uncertainty specification: Not a real parameter choice")))
    }

###############
### 4) Both ###
############### 
   } else {
     
   ## Run over a few parameter values to emulate uncertainty. Only allow all or no uncertainty. 
    ## Just a few too many options otherwise. Can explore single pieces with one or the other off individually anyway    
     ## !*! Again, this is a copout for now, expand this later
     
if (input$dis_type == "None") {
     
    epi.params <- c(
      mu_h     = mu_h
    , B        = tp.B  
    , beta_mh  = tp.beta_mh
    , phi      = tp.phi
    , k        = input$k
    , eta_a    = tp.eta_a  
    , mu_a     = 1 - tp.mu_a
    , beta_hm  = tp.beta_hm 
    , mu_m     = tp.mu_m     ## Mosquitoes blow up in this model with the lab parameters so adjust a bit (drop by 50%)
    , m_per_h  = input$m_per_h
    , eta_h    = input$eta_h     
    , gamma_m  = input$gamma_m
    , gamma_h  = input$gamma_h
    , alpha_h  = input$alpha_h
    , psi_h    = input$psi_h
    , omega_Ah = input$omega_Ah
    , omega_Eh = as.numeric(input$omega_Eh)
    , omega_Em = as.numeric(input$omega_Em)
    , delta_h  = input$delta_h
    )    
    
} else {
  
    epi.params <- c(
      mu_h     = mu_h
    , B        = tp.B  
    , beta_mh  = tp.beta_mh
    , phi      = tp.phi
    , k        = input$k
    , eta_a    = tp.eta_a  
    , mu_a     = 1 - tp.mu_a
    , beta_hm  = tp.beta_hm 
    , mu_m     = tp.mu_m     ## Mosquitoes blow up in this model with the lab parameters so adjust a bit (drop by 50%)
    , m_per_h  = input$m_per_h
    , eta_h    = eta_h     
    , gamma_m  = gamma_m
    , gamma_h  = gamma_h
    , alpha_h  = alpha_h
    , psi_h    = psi_h
    , omega_Ah = omega_Ah
    , omega_Eh = omega_Eh
    , omega_Em = omega_Em
    , delta_h  = delta_h
    )    
  
}
     
    epi.params <- as.data.frame(t(epi.params))
    
    ## Implement the intervention 
    ## *!* could consider some types of intervention that start later in the epidemic
    if (input$intervention != "None") {
        if (input$intervention == "mu_a" | input$intervention == "mu_m" | input$intervention == "eta_h") {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] / int.strength[j]
      } else {
    epi.params[[input$intervention]] <- epi.params[[input$intervention]] * int.strength[j]  
      }
    }
    
    ## Control of uncertainty/parameter variability ########
    if (input$uncer_type == "all") {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs * ncol(epi.params)), ]
    epi.params <- apply(epi.params, 2, function(x) rlnorm(length(x), log(x), input$uncer_logsd * x))
  
    } else if (input$uncer_type %in% names(epi.params) == TRUE) {
      
    epi.params <- epi.params[rep(row.names(epi.params), input$uncer_runs), ]
    ## !*! Placeholder for now for variability/uncertainty
    epi.params[[input$uncer_type]] <- rlnorm(input$uncer_runs, log(epi.params[[input$uncer_type]])
      , input$uncer_logsd * epi.params[[input$uncer_type]])
    
    } else if (input$uncer_type == "none") {
      ## Do nothing      
    } else {
      return(list(c("ERROR with uncertainty specification: Not a real parameter choice")))
    }

   }
     
       epi.params.all[[j]] <- epi.params
     
}  

  ## Possibly will want to support uncertainty in starting values, but for now just make sure the length of the starting values
 ## has the same dimensions as the parameter values

### First set up a list of all the levels of any intervention
epi.start.all <- vector("list", length = length(int.strength))

##########################################
### Package all of the starting values ###
##########################################
### Most efficient to just wrap everything into a loop over the intervention options
for (j in seq_along(epi.start.all)) {

epi.start <- c(
  S_h  = S_h
, E_h  = 0
, I_h  = I_h
, I_Ah = 0
, R_h  = R_h
, N_h  = N_h
, A_m  = if (input$uncer_type == "all") {
  unname(epi.params.all[[j]][attr(epi.params.all[[j]], "dimnames")[[2]] == "k"][1] * N_h)
} else {
  epi.params.all[[j]]$k[1] * N_h 
  }
, S_m = input$m_per_h * N_h
, E_m = 0
, I_m = 0
    )

epi.start <- as.data.frame(t(epi.start))

    ## Control of uncertainty/parameter variability ########
    if (uncer.start == "all") {
      return("ERROR in implementation: Not implemented yet")
    } else if (uncer.start == "none") {
      epi.start  <- epi.start[rep(row.names(epi.start), nrow(epi.params.all[[j]])), ]
     if (input$uncer_type == "all") {
      epi.start$A_m <- unname(epi.params.all[[j]][attr(epi.params.all[[j]], "dimnames")[[2]] == "k"] * N_h)
     } else {
      epi.start$A_m <- epi.params.all[[j]]$k * N_h
     }
    } else {
      return("ERROR in establishing starting conditions: Not a real parameter choice")     
    }

  epi.start.all[[j]] <- epi.start

}

p            <- 1
total.length <- length(epi.params.all) * nrow(epi.params.all[[1]] + 1)
  
###################
### Run the ode ###
###################
  withProgress(message = 'Running ODEs', value = 0, {
     ## Loop over the parameter values to show uncertainty (probably will need to extend when I actually do a better 
      ## job with the uncertainty)
  for (j in seq_along(epi.params.all)) {
    for (i in 1:nrow(epi.params.all[[j]])) {
    
    ### Run the ode
    epi.out.t <- as.data.frame(
  lsoda(
    y     = unlist(epi.start.all[[j]][i, ])
  , times = seq(0, np, by = 1)
  , parms = epi.params.all[[j]][i, ]
  , func  = vector_epi_model_expanded
    )
  ) %>% 
      mutate(
        run          = i
      , intervention = int.strength[j]) 
    
      if (i == 1 & j == 1) {
    epi.out <- epi.out.t
  } else {
    epi.out <- rbind(epi.out, epi.out.t)
  }
    
  incProgress(1/total.length, detail = paste("working on run:", p))
  p <- p + 1
    
  }
}
    
  })
  
  ### clean up for plotting
    epi.out.gg   <- epi.out %>% melt(id = c("time", "run", "intervention"))
    epi.out.gg.h <- epi.out.gg %>% 
      filter(variable == "S_h" | variable == "E_h" | variable == "I_h" | variable == "I_Ah" | variable == "R_h") %>%
      droplevels()
    epi.out.gg.m <- epi.out.gg %>% 
      filter(variable == "A_m" | variable == "E_m" | variable == "S_m" | variable == "I_m") %>%
      droplevels()
    
  list(
    epi.out      = epi.out
  , epi.params   = epi.params
  , epi.out.gg.h = epi.out.gg.h
  , epi.out.gg.m = epi.out.gg.m
    )
  
   })
  
  output$graph1 <- renderPlot({
  
  if (max(epi.dat()[["epi.out.gg.h"]]$run) == 1) {
    
  gg.epi1 <- ggplot(epi.dat()[["epi.out.gg.h"]], aes(time, log(value))) + 
  geom_line(aes(colour = variable), lwd = 2) +
  scale_y_continuous(labels = function(x) format(x, scientific = TRUE)) +
   scale_color_brewer(
    palette = "Dark2"
  , name    = "State Class"
  , labels  = c(
    "Susceptible Hosts"
  , "Exposed Hosts"
  , "Infected Hosts"
  , "Asymptomatic Hosts"
  , "Recovered Hosts")) +
  xlab("Time") + ylab("Count") +
    facet_wrap(~intervention)
  
   } else {
     
epi.out.gg.h.s <- epi.dat()[["epi.out.gg.h"]] %>%
  group_by(variable, time, intervention) %>%
  summarize(
    est = quantile(value, 0.50)
  , lwr = quantile(value, 0.10)
  , upr = quantile(value, 0.90)
  )
epi.out.gg.h.s <- droplevels(epi.out.gg.h.s)
     
  gg.epi1 <- ggplot(epi.out.gg.h.s, aes(time, log(est))) + 
  geom_line(aes(colour = variable), lwd = 2) +
  geom_ribbon(aes(time, ymin = log(lwr), ymax = log(upr), fill = variable), alpha = 0.5) +
  scale_y_continuous(labels = function(x) format(x, scientific = TRUE)) +
   scale_color_brewer(
    palette = "Dark2"
  , name    = "State Class"
  , labels  = c(
    "Susceptible Hosts"
  , "Exposed Hosts"
  , "Infected Hosts"
  , "Asymptomatic Hosts"
  , "Recovered Hosts")) +
   scale_fill_brewer(
    palette = "Dark2"
  , name    = "State Class"
  , labels  = c(
    "Susceptible Hosts"
  , "Exposed Hosts"
  , "Infected Hosts"
  , "Asymptomatic Hosts"
  , "Recovered Hosts")) +
  xlab("Time") + ylab("Count") +
    facet_wrap(~intervention)    
     
   }
    
  print(gg.epi1)
    
  })
  
  output$graph2 <- renderPlot({
    
   if (max(epi.dat()[["epi.out.gg.m"]]$run) == 1) {
    
  gg.epi2 <- ggplot(epi.dat()[["epi.out.gg.m"]], aes(time, log(value))) + 
  geom_line(aes(colour = variable), lwd = 2) +
  scale_y_continuous(labels = function(x) format(x, scientific = TRUE)) +
   scale_color_brewer(
    palette = "Dark2"
  , name    = "State Class"
  , labels  = c(
    "Aquatic Mosquitoes"
  , "Susceptible Mosquitoes"
  , "Exposed Mosquitoes"
  , "Infected Mosquitoes")) +
  xlab("Time") + ylab("Count") +
    facet_wrap(~intervention)
  
   } else {
     
epi.out.gg.m.s <- epi.dat()[["epi.out.gg.m"]] %>%
  group_by(variable, time, intervention) %>%
  summarize(
    est = quantile(value, 0.50)
  , lwr = quantile(value, 0.10)
  , upr = quantile(value, 0.90)
  )

epi.out.gg.m.s <- droplevels(epi.out.gg.m.s)
     
  gg.epi2 <- ggplot(epi.out.gg.m.s, aes(time, log(est))) + 
  geom_line(aes(colour = variable), lwd = 2) +
  geom_ribbon(aes(time, ymin = log(lwr), ymax = log(upr), fill = variable), alpha = 0.5) +
  scale_y_continuous(labels = function(x) format(x, scientific = TRUE)) +
   scale_color_brewer(
    palette = "Dark2"
  , name    = "State Class"
  , labels  = c(
    "Aquatic Mosquitoes"
  , "Susceptible Mosquitoes"
  , "Exposed Mosquitoes"
  , "Infected Mosquitoes")) +
   scale_fill_brewer(
    palette = "Dark2"
  , name    = "State Class"
  , labels  = c(
    "Aquatic Mosquitoes"
  , "Susceptible Mosquitoes"
  , "Exposed Mosquitoes"
  , "Infected Mosquitoes")) +
  xlab("Time") + ylab("Count") + 
    facet_wrap(~intervention)
     
   }
    
  print(gg.epi2)
    
  })
  
  output$debug <- renderPrint({
  epi.dat()
  })
  
  output$summary <- renderPrint({
    head(epi.dat()[["epi.out"]])
  })
  
 output$datadown <- downloadHandler(

    # This function returns a string which tells the client
    # browser what name to use when saving the file.
    filename = function() {
		  paste(paste("epi.out", paste(strsplit(as.character(Sys.time()), split = " +")[[1]], collapse = "_"), sep = "_"), "csv", sep = ".")
	  },

    # This function should write data to a file given to it by
    # the argument 'file'.
    content = function(file) {
      write.csv(epi.dat()[["epi.out"]], file, row.names = FALSE)
    }
  )
 
}
